// Ionic Starter App

// angular.module is a global place for creating, registering and retrieving Angular modules
// 'starter' is the name of this angular module example (also set in a <body> attribute in index.html)
// the 2nd parameter is an array of 'requires'
angular.module('tribe', [
  'ionic','ionic.service.core',
  //'ion-gallery',
  'ngCordova',
  'angular-carousel',
  'ionic.service.analytics',
  'ngResource',
  'ngSpecialOffer',
  'ngStorage',
  'ionic.ion.imageCacheFactory',
  'tribe.services',
  'tribe.controllers',
  'tribe.directives',
  'angulartics', 
  'angulartics.google.analytics'
])

.run(function($ionicPlatform, $rootScope, $ionicConfig, $timeout, $cordovaLocalNotification, $state,
              $cordovaGeolocation, $cordovaStatusbar, $ionicHistory, $ionicNavBarDelegate, $ionicAnalytics, $specialOffer, SettingService, FacebookService) {
  $ionicPlatform.ready(function() {
    
    $ionicAnalytics.register();

    // Hide the accessory bar by default (remove this to show the accessory bar above the keyboard
    // for form inputs)
    if (window.cordova && window.cordova.plugins.Keyboard) {
      cordova.plugins.Keyboard.hideKeyboardAccessoryBar(true);
    }
    /*if (window.StatusBar) {
      //StatusBar.styleDefault();
      $cordovaStatusbar.hide(); //Let's just hide the status bar, it's annoying.
    }*/


    FacebookService.checkLoginStatus();

    var appVersion = '1.0.0';
    var iosId = '1078961862';
    var androidPackageName = 'com.tribeptyltd.tribe';
    $specialOffer.init({
      id           : 'rate-tribe-v' + appVersion,
      showOnCount  : 5,
      title        : 'Rate Tribe',
      text         : 'Like Tribe? Climb a big hill and shout it from the mountaintops! ' +
                     '\nAlternatively, just take a moment to rate us.',
      agreeLabel   : 'Rate App',
      remindLabel  : 'Later',
      declineLabel : 'Not Interested',
      onAgree      : function () {
        if (window.device.platform === 'iOS') {
          window.open($specialOffer.appStoreUrl(iosId));
        } else if (window.device.platform === 'Android') {
          window.open($specialOffer.googlePlayUrl(androidPackageName));
        }
      },
      onDecline   : function () {
        // declined
      },
      onRemindMeLater : function () {
        // will be reminded in 5 more uses
      }
    });

  });

  // This fixes transitions for transparent background views
  $rootScope.$on("$stateChangeStart", function(event, toState, toParams, fromState, fromParams){
    if(toState.name.indexOf('auth.intro') > -1)
    {
      // set transitions to android to avoid weird visual effect in the walkthrough transitions
      $timeout(function(){
        $ionicConfig.views.transition('android');
        $ionicConfig.views.swipeBackEnabled(false);
        console.log("setting transition to android and disabling swipe back");
      }, 0);
    }
  });

  $rootScope.$on("$stateChangeSuccess", function(event, toState, toParams, fromState, fromParams){
    console.log('$ionicHistory', $ionicHistory.viewHistory());
    // Set theme color
    var tabThemes = {
      'app.tabbed.curated': 'red',
      'app.tabbed.discover': 'orange',
      'app.tabbed.myTribe': 'yellow'
    };
    if (tabThemes[toState.name]) {
      $rootScope.theme = tabThemes[toState.name];
    }
  });

  $ionicPlatform.registerBackButtonAction (function(event) {
    event.preventDefault();
    event.stopPropagation();
    if ($state.is('app.tabbed.curated')) {
      ionic.Platform.exitApp();
      return false;
    }
    if ($state.is('app.tabbed.discover') || $state.is('app.tabbed.myTribe')) {
      $state.go('app.tabbed.curated');
      return false;
    }
    $ionicHistory.goBack();
    return false;
  }, 101);

  $rootScope.goTo = function(stateName) {
    $state.go(stateName);
  };

  // Set initial theme
  $rootScope.theme = 'red';
})

.config(function($stateProvider, $urlRouterProvider, $ionicConfigProvider) {

    // Remove text on nav bar's back button
    $ionicConfigProvider.backButton.previousTitleText(false).text('');

    $stateProvider

      .state('auth', {
        url: "/auth",
        templateUrl: "views/auth/auth.html",
        abstract: true,
      })

      .state('auth.intro', {
        url: '/intro',
        templateUrl: "views/auth/intro.html",
        controller: 'FacebookloginCtrl',
      })

      .state('app', {
        url: '/app',
        templateUrl: 'views/app/app.html',
        controller: 'AppCtrl',
        abstract: true
      })

      .state('app.tabbed', {
        templateUrl: 'views/app/tabbed.html',
        controller: 'TabbedCtrl',
        abstract: true,
      })

      .state('app.tabbed.curated', {
        url: '/curated',
        views: {
          'curated-tab': {
            templateUrl: 'views/app/curated.html'
          }
        }
      })

      .state('app.tabbed.discover', {
        url: '/discover',
        views: {
          'discover-tab': {
            templateUrl: 'views/app/discover.html'
          }
        }
      })

      .state('app.tabbed.myTribe', {
        url: '/mytribe',
        views: {
          'mytribe-tab': {
            templateUrl: 'views/app/mytribe.html'
          }
        }
      })

      .state('app.tabbed.curatedEvent', {
        url: '/curated/:id',
        views: {
          'curated-tab@app.tabbed': {
            templateUrl: 'views/app/event-details.html'
          },
        },
        controller: 'EventDetailsCtrl'
      })

      .state('app.tabbed.curatedGallery', {
        url: '/curated/gallery/:id',
        views: {
          'curated-tab@app.tabbed': {
            templateUrl: 'views/app/gallery.html'
          },
        },
        controller: 'GalleryCtrl'
      })

      .state('app.tabbed.discoverEvent', {
        url: '/discover/:id',
        views: {
          'discover-tab@app.tabbed': {
            templateUrl: 'views/app/event-details.html'
          },
        },
        controller: 'EventDetailsCtrl'
      })

      .state('app.tabbed.discoverGallery', {
        url: '/discover/gallery/:id',
        views: {
          'discover-tab@app.tabbed': {
            templateUrl: 'views/app/gallery.html'
          },
        },
        controller: 'GalleryCtrl'
      })

      .state('app.tabbed.myTribeEvent', {
        url: '/mytribe/:id',
        views: {
          'mytribe-tab@app.tabbed': {
            templateUrl: 'views/app/event-details.html'
          },
        },
        controller: 'EventDetailsCtrl'
      })

      .state('app.tabbed.myTribeGallery', {
        url: '/mytribe/gallery/:id',
        views: {
          'mytribe-tab@app.tabbed': {
            templateUrl: 'views/app/gallery.html'
          },
        },
        controller: 'GalleryCtrl'
      })

      .state('app.eventDetail', {
        url: '/event/:id',
        templateUrl: 'views/app/event-details.html',
        controller: 'EventDetailsCtrl'
      })

      .state('app.tabbed.category', {
        url: '/category/:id',
        views: {
          'discover-tab@app.tabbed': {
            templateUrl: 'views/app/category.html'
          },
        },
      })

      .state('app.tabbed.curated-settings', {
        url: '/curated/settings',
        views: {
          'curated-tab': {
            templateUrl: 'views/app/settings.html',
            controller: 'SettingsCtrl'
          }
        },
      })

      .state('app.tabbed.discover-settings', {
        url: '/discover/settings',
        views: {
          'discover-tab': {
            templateUrl: 'views/app/settings.html',
            controller: 'SettingsCtrl'
          }
        },
      })

      .state('app.tabbed.curated-settings-city', {
        url: '/curated/city',
        views: {
          'curated-tab': {
            templateUrl: 'views/app/settings.city.html',
          }
        }
      })

      .state('app.tabbed.discover-settings-city', {
        url: '/discover/city',
        views: {
          'discover-tab': {
            templateUrl: 'views/app/settings.city.html',
          }
        }
      })
    ;

    $urlRouterProvider.otherwise('/curated');

    $ionicConfigProvider.navBar.alignTitle('center');
  });

document.addEventListener("deviceready", onDeviceReady, false);
function onDeviceReady() {
    console.log("Status Bar: " + StatusBar);
    StatusBar.overlaysWebView(false);
}
