angular.module('tribe.services', [
    'ionic',
    'ngCordova',
    'ionic.service.core',
  ])

  .service('UserService', function ($http, $q) {

    var currentUser = {interested_events: []};

    var setUser = function (user_data) {
      var deferred = $q.defer();
      // Post Facebook user data to server and receive Tribe user data
      $http({
        method: 'POST',
        url: 'http://54.153.195.194/api/user/facebook',
        data: user_data
      }).then(function (response) {
        currentUser = response.data.data;
        currentUser.fbUser = user_data;
        if (!currentUser.interested_events)
          currentUser.interested_events = [];

        //window.localStorage.setItem('user', JSON.stringify(currentUser));
        deferred.resolve(currentUser);
      });
      return deferred.promise;
    };

    var getUser = function () {
      //if (currentUser == null) {
      //  currentUser = JSON.parse(window.localStorage.getItem('user'));
      //}
      return currentUser;
    };

    return {
      getUser: getUser,
      setUser: setUser
    };
  })

  // Retrieve data from Tribe API and store in device's memory
  .service('TribeService', function ($http, $q, $rootScope, $cordovaLocalNotification, SettingService, UserService) {

    var host = 'http://54.153.195.194/';
    var categories, events, states; // Memory cache

    function settingsToParams(settings, params) {
      if (params == null) {
        params = {};
      }
      if (settings.city != 'All') {
        params.city = settings.city;
      }
      params.familyEvents = settings.familyEvents;
      params.age = settings.age;
      params['category'] = [];
      // If categories are not set in Settings, set to allow all categories
      if (Object.keys(settings.category).length == 0) {
        for (var i = 0; i < 10; i++) {
          settings.category[i] = true;
        }
      }
      for (var i in settings.category) {
        if (settings.category[i]) {
          params['category'].push(i);
        }
      }
      params['friends'] = settings.friends;
      return params;
    }

    var getCategories = function(fromCache) {
      if (fromCache == null) {
        fromCache = true;
      }
      var deferred = $q.defer();
      if (fromCache && categories) {
        deferred.resolve(categories);
      } else {
        $http({
          method: 'GET',
          cache: true,
          url: host + 'api/dict/category'
        }).then(function (response) {
          categories = response.data;
          deferred.resolve(response.data);
        });
      }
      return deferred.promise;
    };

    var getCategory = function (id) {
      var deferred = $q.defer();
      if (categories) {
        deferred.resolve(categories[id]);
      } else {
        getCategories().then(function (data) {
          deferred.resolve(data[id]);
        });
      }
      return deferred.promise;
    }

    var getEvents = function (params, fromCache, loadSettings) {
      if (fromCache == null) {
        fromCache = true;
      }
      if (loadSettings == null) {
        loadSettings = true;
      }
      var deferred = $q.defer();
      if (fromCache && events) {
        deferred.resolve(events);
      } else {
        $http({
          method: 'POST',
          url: host + 'api/event',
          data: loadSettings ? settingsToParams(SettingService.get(), params) : params,
        }).then(function (response) {
          var newEvents = response.data;
          var user = UserService.getUser();
          for (var id in newEvents) {
            if (user.interested_events.indexOf(id) > -1) {
              newEvents[id].isInterested = true;
            } else {
              newEvents[id].isInterested = false;
            }
          }
          // Save to cache only when received the full list of events
          if (!params && !loadSettings) {events = newEvents; console.log('saved to cache');}
          deferred.resolve(newEvents);
        });
      }
      return deferred.promise;
    };

    var getEvent = function (id) {
      var deferred = $q.defer();
      if (events && events[id]) {
        deferred.resolve(events[id]);
      } else {
        $http({
          method: 'POST',
          url: host + 'api/event/' + id,
          data: {
            friends: SettingService.get('friends')
          }
        }).then(function (response) {
          deferred.resolve(response.data);
        });
      }
      return deferred.promise;
    };

    var getStates = function(fromCache) {
      if (fromCache == null) {
        fromCache = true;
      }
      var deferred = $q.defer();
      if (fromCache && states) {
        deferred.resolve(states);
      } else {
        $http({
          method: 'GET',
          url: host + 'api/dict/state',
        }).then(function (response) {
          states = response.data;
          deferred.resolve(response.data);
        });
      }
      return deferred.promise;
    };

    var fetchAll = function (callback) {
      $q.all([
        getCategories(false),
        getEvents(null, false),
        getStates(false),
      ]).then(function (data) {
        if (typeof callback == 'function') {
          callback(data);
        }
      });
    };

    var removeFromMyTribe = function(event) {
      console.log('remove from my tribe', event.id);
      var user = UserService.getUser();
      $http({
        method: 'POST',
        url: host + 'api/user/' + user.id + '/mytribe',
        data: {action: 'remove', event_id: event.id}
      }).then(function(response) {
        var i = user.interested_events.indexOf(event.id);
        delete user.interested_events[i];
        $rootScope.$broadcast('RemovedFromMyTribe');
      });
      // Cancel the notification
      $cordovaLocalNotification.cancel(event.id, $rootScope);
    };

    var addToMyTribe = function(event) {
      console.log('add to my tribe', JSON.stringify(event));
      var user = UserService.getUser();
      $http({
        method: 'POST',
        url: host + 'api/user/' + user.id + '/mytribe',
        data: {action: 'add', event_id: event.id}
      }).then(function(response) {
        console.log('response', JSON.stringify(response));
        user.interested_events.push(event.id);

        // Schedule a reminder notification
        var settings = SettingService.get();
        if (settings.showNotification) {
          var messages = [
            'The Tribe gathers tomorrow at [event]',
            'Reminder: the Tribe are hanging out tomorrow at [event]',
            'Pause your wandering, friend! [event] is tomorrow!'
          ];
          var text = messages[Math.floor((Math.random() * 3))];
          text = text.replace('[event]', event.title);
          console.log('notification text', text);
          var alarmTime = moment.unix(event.start_at_unix).subtract(1,'day');
          alarmTime.set('minute',0);
          alarmTime.set('second',0);
          console.log('alarmTime', alarmTime.format());
          $cordovaLocalNotification.schedule({
            id: event.id,
            at: alarmTime.toDate(),
            title: "Tribe",
            text: text,
            sound: null,
            icon:      'icon',
            smallIcon: 'icon',
          }).then(function () {
            console.log("The notification has been set");
          });
        }
        $rootScope.$broadcast('AddedToMyTribe');
      });
    };

    return {
      getCategories: getCategories,
      getCategory: getCategory,
      getEvents: getEvents,
      getEvent: getEvent,
      getStates: getStates,
      fetchAll: fetchAll,
      removeFromMyTribe: removeFromMyTribe,
      addToMyTribe: addToMyTribe,
    };
  })

  .service('SettingService', function ($rootScope) {

    var settings = null;

    var set = function() {
      if (arguments.length == 1) { // set the whole settings object
        settings = arguments[0];
      } else { // set one setting value
        settings = get();
        settings[arguments[0]] = arguments[1];
      }
      //if (!settings.showNotification) {
      //  $cordovaLocalNotification.clearAll($rootScope);
      //}
      window.localStorage.setItem('settings', JSON.stringify(settings));
    };
    var get = function() {
      if (settings == null) {
        // If settings is not in memory, load from local storage
        settings = JSON.parse(window.localStorage.getItem('settings'));
        if (settings == null) {
          // Load the default setting
          settings = {
            city: 'All',
            familyEvents: true,
            showNotification: true,
            age: null,
            category: {}
          };
        }
      }

      if (arguments.length == 0) {
        return settings;
      } else {
        return settings[arguments[0]];
      }
    };
    var getTutorial = function(name) {
      var tutorials = JSON.parse(window.localStorage.getItem('tutorial'));
      if (tutorials == null) {
        tutorials = {
          'curated-addToMyTribe': true,
          'eventDetails-addToMyTribe': true,
          'curated-introduction': true,
          'discover-introduction': true,
          'mytribe-introduction': true,
        };
      }
      if (name == null) {
        return tutorials;
      }
      return tutorials[name];
    };
    var setTutorial = function(name, value) {
      var tutorials = getTutorial();
      tutorials[name] = value;
      window.localStorage.setItem('tutorial', JSON.stringify(tutorials));
    };
    var getAddedToCalendar = function(eventId) {
      var addedEvents = JSON.parse(window.localStorage.getItem('calendar-events'));
      if (addedEvents == null) {
        addedEvents = [];
      }
      if (eventId == null) {
        return addedEvents;
      }
      return addedEvents.indexOf(eventId) > -1;
    };
    var setAddedToCalendar = function(eventId) {
      var addedEvents = getAddedToCalendar();
      if (addedEvents.indexOf(eventId) == -1) {
        addedEvents.push(eventId);
      }
      window.localStorage.setItem('calendar-events', JSON.stringify(addedEvents));
    };
    return {
      get: get,
      set: set,
      getTutorial: getTutorial,
      setTutorial: setTutorial,
      getAddedToCalendar: getAddedToCalendar,
      setAddedToCalendar: setAddedToCalendar,
    };
  })

  //.service('FacebookService', function($state, $q, UserService, SettingService, $ionicLoading, $cordovaLocalNotification, $cordovaGeolocation, $cordovaFacebook) {
  .service('FacebookService', function($state, $q, UserService, SettingService, TribeService, $ionicLoading, $cordovaGeolocation, $cordovaFacebook, $cordovaLocalNotification, $rootScope, $http) {

    // This is the success callback from the login method
    var fbLoginSuccess = function (response) {
      if (!response.authResponse) {
        fbLoginError("Cannot find the authResponse");
        return;
      }

      var authResponse = response.authResponse;

      getFacebookProfileInfo(authResponse)
        .then(function (profileInfo) {
          // Store user data on local storage - add to Db storage
          UserService.setUser({
            authResponse: authResponse,
            userID: profileInfo.id,
            name: profileInfo.name,
            email: profileInfo.email,
            birthday: profileInfo.birthday,
            picture: "http://graph.facebook.com/" + authResponse.userID + "/picture?type=large"
          }).then(function(currentUser) {
            $ionicLoading.hide();
            $state.go('app.tabbed.curated');
          });

          // Store user age to Settings
          var birthday = moment(profileInfo.birthday);
          if (birthday.isValid()) {
            SettingService.set('age', moment().diff(birthday, 'years'));
          }

          // Get list of friends that installed the app
          facebookConnectPlugin.api("/me/friends?fields=picture,name,email,installed", ["public_profile", "user_friends"],
            function (result) {
              console.log("Friends Result: " + JSON.stringify(result));
              var friendIds = [];
              for (var i in result.data) {
                friendIds.push(result.data[i].id);
              }
              SettingService.set('friends',friendIds);
            },
            function (error) {
              console.log("Get Friends Failed: " + error);
            }
          );
        }, function (fail) {
          // Fail get profile info
          console.log('profile info fail', fail);
          $ionicLoading.hide();
        });


      // Get the current location
      console.log('getting location');
      var position = $cordovaGeolocation.getCurrentPosition().then(function (position) {
        console.log('position', position);
        var coords = position.coords;
        $http({
          method: 'GET',
          url: 'http://maps.googleapis.com/maps/api/geocode/json?sensor=false',
          params: {
            latlng: coords.latitude + ',' + coords.longitude
          }
        }).then(function (response) {
          console.log('location', JSON.stringify(response));
          var state = response.data.results[0].address_components[3].short_name;
          // Set state to settings
          TribeService.getStates().then(function(states) {
            for (var stateId in states) {
              if (state == stateId) {
                SettingService.set('city',states[stateId].city);
              }
              return;
            }
            SettingService.set('city','All');
          });
        });
      }, function(error) {
        console.log('location error', JSON.stringify(error));
      });

    };

    // This is the fail callback from the login method
    var fbLoginError = function (error) {
      console.log('fbLoginError', JSON.stringify(error));
      $ionicLoading.hide();
      $state.go('auth.intro');
    };

    // This method is to get the user profile info from the facebook api
    var getFacebookProfileInfo = function (authResponse) {
      var info = $q.defer();

      facebookConnectPlugin.api('/me?fields=email,first_name,last_name,birthday,gender&access_token=' + authResponse.accessToken, null,
        function (response) {
          console.log(response);
          info.resolve(response);
        },
        function (response) {
          console.log(response);
          info.reject(response);
        }
      );
      return info.promise;
    };

    var login = function() {
      // Ask the permissions we need. See documentation -
      // FB permissions here: https://developers.facebook.com/docs/facebook-login/permissions/v2.4
      facebookConnectPlugin.login(['email', 'public_profile', 'user_birthday', 'user_friends'], fbLoginSuccess, fbLoginError);

      // Ask for Notification permission on iOS
      if(window.ionic.Platform.is("iOS")) {
        $cordovaLocalNotification.promptForPermission();
      }
    }

    var checkLoginStatus = function () {

      if (window.ionic.Platform.is('browser')) {
        $state.go('app.tabbed.curated');
      } else {
        $ionicLoading.show({
          template: 'Loading...'
        });
        window.facebookConnectPlugin.getLoginStatus(function (success) {
          if (success.status === 'connected') {
            // The user is logged in and has authenticated tribe, and response.authResponse supplies
            // the user's ID, a valid access token, a signed request, and the time the access token
            // and signed request each expire
            console.log('getLoginStatus', success.status);

            getFacebookProfileInfo(success.authResponse)
              .then(function (profileInfo) {

                console.log('profileInfo', JSON.stringify(profileInfo));

                // Store user data on local storage - add to Db storage
                UserService.setUser({
                  authResponse: success.authResponse,
                  userID: profileInfo.id,
                  first_name: profileInfo.first_name,
                  last_name: profileInfo.last_name,
                  email: profileInfo.email,
                  gender: profileInfo.gender,
                  birthday: profileInfo.birthday,
                  picture: "http://graph.facebook.com/" + success.authResponse.userID + "/picture?type=large"
                }).then(function () {
                  $ionicLoading.hide();
                  $state.go('app.tabbed.curated');
                });

              });

          } else {
            // If (success.status === 'not_authorized') the user is logged in to Facebook,
            // but has not authenticated tribe
            // Else the person is not logged into Facebook,
            // so we're not sure if they are logged into tribe or not.

            console.log('getLoginStatus', success.status);
            $ionicLoading.show({
              template: 'Logging in...'
            });
            login();
          }
        });
      }
    }

    var logout = function () {

      $ionicLoading.show({
        template: 'Logging out...'
      });

      //facebook logout
      facebookConnectPlugin.logout(function () {
          $ionicLoading.hide();
          $state.go('auth.intro');
        },
        function (fail) {
          $ionicLoading.hide();
        });
    };

    return {
      getFacebookProfileInfo: getFacebookProfileInfo,
      checkLoginStatus: checkLoginStatus,
      login: login,
      logout: logout
    };
  })
;
